	program exampleC2

!
!	Program solves singularly perturbed differential two-point boundary
!	value  problem:
!	ey"+y'-(1+e)*y=0, y(0)=1+exp(-1), y(1)=1+exp(-(1+e)/e, e<<1
!	by the collocation method by tension spline at the knots of the
!	partition of [0,1] (see Example 8.2)
!

	use dfport
	use examplediffeqC2

	implicit none

	integer :: n,st,io,i,j,ileft
	double precision, allocatable :: xx(:),px(:),c(:),c1(:),c2(:)
	double precision :: eps,h,x,spl,spl1der,spl2der,sol,err,maxerr

	print 10
 10	format ("******************************************************************************",/, &
			"*                                                                            *",/, &
			"* Program solves singularly perturbed differential two-point boundary value  *",/, &
			"* problem:", &
			" e*y","''","+y","'","-(1+e)*y=0, y(0)=1+exp(-1), y(1)=1+exp(-(1+e)/e, e<<1     *",/, &
			"* from Example 8.2 with tension splines                                      *",/, &
			"*                                                                            *",/, &
			"******************************************************************************")
!
!	This example is calculated for the leading parameter in the 
!	differential equation e=2.0d0**(-6), and the number of 
!	subintervals n=2**6
!
	eps=2.0d0**(-6)
	print '(2/,"The parameter epsilon:",d20.15)', eps
	n=2**6
	print '(/,"The number of subintervals:",i7,2/)',n
!
!	Allocation of the arrays that will contain knots (xx), tension 
!	parameters (px), de Boor points of the Chebyshev tension spline (c),
!	de Boor points of the first (c1) and the second (c2) derivative of 
!	the Chebyshev tension spline
!
	allocate (xx(n+7),px(n+6),c(n+3),c1(n+3),c2(n+3),stat=st)
	if (st/=0) then
		print '("Allocation was unsuccessful!")'
		stop
	endif
!
!	Solving the system of linear equations, comming from the collocation
!	method (calculating the de Boor poits of the tension spline (c))
!
	call linsystemC2 (eps,n,xx,px,c)
!
!	Calculating the de Boor points of the first (c1) and the second (c2)
!	derivative of the tension spline
!
	c1=c
	call tension1der4C2coeff (xx,px,c1,n+3,1)
	c2=c1
	call tension1der3C2coeff (xx,px,c2,n+3,1)
!
!	The file "exampleC2" will contain the value of the exact solution,
!	the value of the tension spline, it's first and second
!	derivative, and the absolute error at each point x
!
	open (1,file="exampleC2",status="unknown",iostat=io)
	if  (io/=0) then
		print '("Opening of the file was unsuccessful!")'
		stop
	endif
	rewind (1)
!
!	Sampling equaidistant points from [0,1]
!
	h=0.1d0/n
	maxerr=0.0d0
	do i=0,n
		x=xx(i+4)
		ileft=i+4
		if (i==n) ileft=ileft-1
!
!	Calculating the value of the tension spline at x
!
		call tension4C2 (x,xx,px,ileft,c,1,spl)
!
!	Calculating the first derivative of the tension spline at x
!
		call tension3C2 (x,xx,px,ileft,c1,1,spl1der)
!
!	Calculating the second derivative of the tension spline at x
!
		call tension2 (x,xx,px,ileft,c2,ileft,1,spl2der)
		sol=solution(x,eps)
		err=abs(spl-sol)
!
!	Calculating the maximal  absolute error
!
		if (err>maxerr) maxerr=err
		write (1,'(6f22.15)') x,sol,spl,spl1der,spl2der,err
	enddo

	deallocate (xx,px,c,c1,c2)
	close (1)

	print '("The largest absolute error on the interval [0,1] is ",d20.15)',maxerr
!
!	Plotting the results with GNUPLOT. For download of GNUPLOT use
!	www.gnuplot.info/download.html,
!	and uncomment next line
!
!	i=system("wgnupl32.exe plotC2.txt")  


	end program exampleC2